//! A library for reading and writing ZIP archives.
//! ZIP is a format designed for cross-platform file "archiving".
//! That is, storing a collection of files in a single datastream
//! to make them easier to share between computers.
//! Additionally, ZIP is able to compress and encrypt files in its
//! archives.
//!
//! The current implementation is based on [PKWARE's APPNOTE.TXT v6.3.9](https://pkware.cachefly.net/webdocs/casestudies/APPNOTE.TXT)
//!
//! ---
//!
//! [`zip`](`crate`) has support for the most common ZIP archives found in common use.
//! However, in special cases,
//! there are some zip archives that are difficult to read or write.
//!
//! This is a list of supported features:
//!
//! |         | Reading | Writing |
//! | ------- | ------  | ------- |
//! | Stored | ✅ | ✅ |
//! | Deflate | ✅ [->](`crate::ZipArchive::by_name`)      | ✅ [->](`crate::write::FileOptions::compression_method`) |
//! | Deflate64 | ✅ | |
//! | Bzip2 | ✅ | ✅ |
//! | ZStandard | ✅ | ✅ |
//! | LZMA | ✅ | |
//! | XZ | ✅ | |
//! | AES encryption | ✅ | ✅ |
//! | ZipCrypto deprecated encryption | ✅ | ✅ |
//!
//!
#![cfg_attr(docsrs, feature(doc_auto_cfg))]
#![warn(missing_docs)]
#![allow(unexpected_cfgs)] // Needed for cfg(fuzzing) on nightly as of 2024-05-06

pub use crate::compression::{CompressionMethod, SUPPORTED_COMPRESSION_METHODS};
pub use crate::read::HasZipMetadata;
pub use crate::read::ZipArchive;
pub use crate::spec::{ZIP64_BYTES_THR, ZIP64_ENTRY_THR};
pub use crate::types::{AesMode, DateTime};
pub use crate::write::ZipWriter;

#[cfg(feature = "aes-crypto")]
mod aes;
#[cfg(feature = "aes-crypto")]
mod aes_ctr;
mod compression;
mod cp437;
mod crc32;
pub mod extra_fields;
pub mod read;
pub mod result;
mod spec;
mod types;
pub mod write;
mod zipcrypto;
pub use extra_fields::ExtraField;

#[doc = "Unstable APIs\n\
\
All APIs accessible by importing this module are unstable; They may be changed in patch \
releases. You MUST use an exact version specifier in `Cargo.toml`, to indicate the version of this \
API you're using:\n\
\
```toml\n
[dependencies]\n
zip = \"="]
#[doc=env!("CARGO_PKG_VERSION")]
#[doc = "\"\n\
```"]
pub mod unstable;

// ====== wasm start ====== //

use std::io::{Cursor, Read};

use result::{ZipResult, ZipError};

use wasm_bindgen::{prelude::*, JsValue};
use js_sys::{Array, Object, Error, Reflect::set};

//
// Utilities
//

impl From<ZipError> for JsValue {
    fn from(err: ZipError) -> JsValue {
        match err {
            ZipError::Io(e) => JsValue::from(Error::new(&format!("IO Error: {}", e))),
            ZipError::InvalidArchive(msg) => JsValue::from(Error::new(&format!("Invalid Archive: {}", msg))),
            ZipError::UnsupportedArchive(msg) => JsValue::from(Error::new(&format!("Unsupported Archive: {}", msg))),
            ZipError::FileNotFound => JsValue::from(Error::new("File Not Found")),
            ZipError::InvalidPassword => JsValue::from(Error::new("Invalid Password")),
        }
    }
}


/// Converts a Rust `Vec` to a JS Array.
fn vec_to_js_array<T: Into<JsValue>>(vec: Vec<T>) -> Array {
    let js_array = Array::new();
    for item in vec {
        js_array.push(&item.into());
    }

    js_array
}

//
// Exports
//

/// Unzips a file provided as a JS array, returning a mapping of filenames to decompressed
/// contents.
#[wasm_bindgen]
pub fn unzip(bytes: &[u8]) -> ZipResult<JsValue> {
    // TODO: Accept Uint8Array instead of array

    let cursor = Cursor::new(bytes);
    let mut archive = ZipArchive::new(cursor)?;
    let extracted_files = Object::new();

    for i in 0..archive.len() {
        let mut file = archive.by_index(i)?;
        let file_name = file.name().to_string();

        // Read the file contents into a Vec<u8>
        let mut contents = Vec::new();
        file.read_to_end(&mut contents)?;

        // Convert to JS values and insert into object
        let js_file_name = JsValue::from(file_name);
        let js_contents = vec_to_js_array(contents);
        
        set(&extracted_files, &js_file_name, &js_contents).unwrap();
    }

    Ok(JsValue::from(&extracted_files))
}


// ====== wasm end ====== //
